import functools
import json
import time
import logging
import threading
from typing import Optional

from assemblyline.common.iprange import RangeTable

import requests


logger = logging.getLogger("assemblyline.vacuum.department_map")


class DepartmentMap:
    UPDATE_INTERVAL = 60 * 60

    @staticmethod
    @functools.cache
    def load(url: Optional[str], init: Optional[str]):
        return DepartmentMap(url, init)

    def __init__(self, url: Optional[str], init: Optional[str]):
        self.url = url
        self.init_data = init
        self.lock = threading.Lock()
        self.table = RangeTable()
        self.update_time = 0
        self._load_department_map()

    def _load_department_map(self):
        # Don't load more than once every 5 seconds
        if time.time() - self.update_time < 5:
            return

        with self.lock:
            # Recheck in case it was updated while waiting for lock
            if time.time() - self.update_time < 5:
                return

            table = RangeTable()

            try:
                if self.init_data:
                    for row in json.loads(self.init_data):
                        if ':' not in row['LOWER'] and ':' not in row['UPPER']:
                            # print(row["LOWER"], row['UPPER'], row['LABEL'])
                            table.add_range(row['LOWER'], row['UPPER'], row['LABEL'])
            except Exception:
                logger.exception("Error parsing department_map_init")

            try:
                if self.url:
                    res = requests.get(self.url, verify=False)
                    res.raise_for_status()

                    for row in res.json():
                        if ':' not in row['LOWER'] and ':' not in row['UPPER']:
                            # print(row["LOWER"], row['UPPER'], row['LABEL'])
                            table.add_range(row['LOWER'], row['UPPER'], row['LABEL'])
            except Exception:
                logger.exception("Error parsing department_map_url")

            self.table = table
            self.update_time = time.time()

    def _refresh_department_map(self):
        if time.time() - self.update_time > self.UPDATE_INTERVAL:
            self._load_department_map()

    def __getitem__(self, ip) -> Optional[str]:
        self._refresh_department_map()
        try:
            return self.table[ip]
        except KeyError:
            self._load_department_map()
            try:
                return self.table[ip]
            except KeyError:
                return None


if __name__ == '__main__':
    departments = DepartmentMap('', None)
    print(departments['48.49.39.100'])



"""
Code for collecting metric data and feeding it into the orchestration framework.

The task of this module is to take data available from metrics across multiple hosts
providing the same service and combine them into general statistics about the service.
"""
import time
from typing import Dict
from collections import namedtuple

Row = namedtuple('Row', ['timestamp', 'busy', 'throughput'])


class Collection:
    def __init__(self, period, ttl=None):
        """
        A buffer for metrics data from multiple instances of multiple services.

        :param period: Expected seconds between updates
        :param ttl: Seconds before a message is dropped from the buffer
        """
        self.period: float = period
        self.ttl: float = ttl or (period * 1.5)
        self.services: Dict[str, Dict[str, Row]] = {}

    def update(self, service, host, busy_seconds, throughput):
        # Load the sequence of data points that
        try:
            hosts = self.services[service]
        except KeyError:
            hosts = self.services[service] = {}

        # Add the new data
        hosts[host] = Row(time.time(), busy_seconds, throughput)

    def read(self, service):
        now = time.time()

        # Load the last messages from this service
        try:
            hosts = self.services[service]
        except KeyError:
            return None

        # Flush out stale messages
        expired = [_h for _h, _v in hosts.items() if now - _v.timestamp > self.ttl]
        for host_name in expired:
            hosts.pop(host_name, None)

        # If flushing got rid of all of our messages our state is 'offline'
        if not hosts:
            return None

        #
        return {
            'instances': len(hosts),
            'duty_cycle': sum(_v.busy for _v in hosts.values())/(len(hosts) * self.period),
        }
